---
title: "AI for healthcare"
author: "Jean Feydy"
institute: ""
topic: ""
theme: "metropolis"
colortheme:
fonttheme:
fontsize: 11pt
linkcolor: black
urlcolor: blue
linkstyle: bold
aspectratio: 169
titlegraphic: 
logo:
date:
section-titles: false
toc: false
---


## Remember this slide from lecture 1?

`\begin{center}`{=latex}

![](images/dataset/dataset_original.pdf){ height=65% }

**Supervised learning**  $=$  **Regression**. \
We look for a formula $\text{F}(\text{x}_\text{1}, \dots, \text{x}_{\text{D}})$ of the D variables\
that best approximates an important quantity ($\heartsuit$).

`\end{center}`{=latex}


## First thing you should do?


`\begin{center}\vspace*{.5cm}`{=latex}

![](images/cakes/cakes.png){ width=95% }

 \
`\LARGE{ Working with~~\textbf{clients}~~<~~\textbf{colleagues}~~<~~\textbf{friends}.}`{=latex}
`\end{center}`{=latex}

## Wake up: get out of the matrix!


`\begin{center}`{=latex}
Data **science** is never done in a vacuum. \
Our (big) spreadsheets are **partial projections** of a complex reality. \
\
What are we trying to achieve? \
What type of information is available? \
What do we already know? \
\
To understand this **context**, you must break the ice with domain experts.\
 \
This is a **continuous**, **time-consuming** and `\BR{enjoyable}`{=latex} process.
`\end{center}`{=latex}

## Today: well-rounded methods for high-quality features

**1. Decision trees -- for `\BR{heterogeneous}`{=latex} data**

  - Greedy training and regularizations.

`\vspace{.15cm}`{=latex}

**2. K-Nearest Neighbors -- a first `\BR{isotropic}`{=latex} method**

  - Euclidean metrics and normalization.

`\vspace{.15cm}`{=latex}

**3. Linear regression -- to estimate `\BR{global}`{=latex} trends**
  
  - Linear, piecewise linear and polynomial regression.


`\vspace{.15cm}`{=latex}

**4. Kernel methods -- specify a `\BR{custom}`{=latex} prior**
  
  - Smoothness, short- and long-range interactions.
  - Nadaraya--Watson--Shepard and Ridge regressions.





# 1. Decision trees


## Expert knowledge is often distilled as a tree


`\begin{center}\vspace*{.25cm}`{=latex}

![](images/trees/clinical.png){ height=70% }

*Transformation of the national **breast cancer guideline*** \
*into data-driven clinical decision trees*, Hendriks et al., 2019

`\end{center}`{=latex}





## Tree models are easy to train with a greedy algorithm


`\begin{center}\vspace*{.0cm}`{=latex}

![](images/trees/splitting.pdf){ height=70% }

**Recursive** splits that stop if improvements < T $~~\Longleftrightarrow~~$ **greedy** minimization of \
$\text{Fit}_{\text{x},\text{y}}(\text{F}) ~+~ \text{Reg}(\text{F}) ~~=~~ \tfrac{\text{1}}{\text{2}} \sum_\text{i} \| \text{F}(\text{x}_\text{i}) - \text{y}_\text{i} \|^\text{2} ~+~ \text{T} \cdot \text{\#Leaves}(\text{F})~.$

`\end{center}`{=latex}


## Two toy regression problems


::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/dataset.png){height=5cm}`\vspace*{.5cm}`{=latex}

**D=1** -- 9 points **x** on the unit interval $[\text{0}, \text{1}]$. \
**D=2** -- 20 points **x** on the unit square $[\text{0}, \text{1}]^\text{2}$. \
\
In both cases: **scalar** output values **y**.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/dataset.png){height=8cm}`\end{center}`{=latex}
::::
:::

## Decision trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_polynomial_d0.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Depth 0:** \
1 constant value. \
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_polynomial_d0.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Decision trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/decision_tree_1.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Depth 1:** \
2 distinct values. \
The model is **piecewise constant**.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/decision_tree_1.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Decision trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/decision_tree_2.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Depth 2:** \
up to 4 distinct values. \
The model follows the **D axes** \
of the feature space.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/decision_tree_2.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Decision trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/decision_tree_3.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Depth 3:** \
up to 8 distinct values. \
We may choose not to use them all\
to limit the **complexity** of the model.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/decision_tree_3.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Decision trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/decision_tree_4.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Depth 4:** \
up to 16 distinct values. \
Starting to clearly **overfit**.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/decision_tree_4.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Decision trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/decision_tree_5.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Depth 5:** \
up to 32 distinct values. \
Starting to clearly **overfit**.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/decision_tree_5.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Decision trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/decision_tree_10.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Depth 10:** \
up to 1,024 distinct values. \
Full **overfit** on both datasets.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/decision_tree_10.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Decision trees - strengths and weaknesses

Decision trees are:

- `\BR{Interpretable.}`{=latex}
- **Easy** to train and deploy.
- **Fast** and CPU-friendly.
- **Robust**:
  - Only use a few columns at a time.
  - Work well with **heterogenous** information.
  - Only rely on the **ordering** of the features.


`\begin{center}`{=latex}

However, trees also **overfit** quickly and produce **blocky** results. \
Regularization methods mitigate these issues, at the cost of **interpretability**.

`\end{center}`{=latex}


## 1st regularization strategy: boosted sequence of trees


`\begin{center}\vspace*{.0cm}`{=latex}

![](images/trees/boosting.pdf){ width=100% }

 \
**Iterative** fits on the prediction residuals with shallow trees. \
Use a **small** learning rate for better regularization: \
$\text{\BR{Residual}}_\text{\,\BR{i}} = \text{y}_\text{i} - \text{\textbf{0.1}}\cdot \sum_\text{k} \text{Tree}_\text{k} (\text{x}_\text{i})~.$

`\end{center}`{=latex}



## 1st regularization strategy: boosted sequence of trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/adaboost_3_1.png){height=5cm}`\vspace*{.5cm}`{=latex}

**1 tree** of depth 3:\
a simple decision tree, \
with moderate complexity.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/adaboost_3_1.png){height=8cm}`\end{center}`{=latex}
::::
:::


## 1st regularization strategy: boosted sequence of trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/adaboost_3_3.png){height=5cm}`\vspace*{.5cm}`{=latex}

**3 trees** of depth 3:\
sum of three simple decision trees, \
fitted iteratively on **residuals**.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/adaboost_3_3.png){height=8cm}`\end{center}`{=latex}
::::
:::



## 1st regularization strategy: boosted sequence of trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/adaboost_3_5.png){height=5cm}`\vspace*{.5cm}`{=latex}

**5 trees** of depth 3:\
sum of five simple decision trees, \
fitted iteratively on **residuals**.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/adaboost_3_5.png){height=8cm}`\end{center}`{=latex}
::::
:::


## 1st regularization strategy: boosted sequence of trees

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/adaboost_3_100.png){height=5cm}`\vspace*{.5cm}`{=latex}

**100 trees** of depth 3:\
sum of a hundred simple decision trees. \
We reach a high training accuracy \
with a relatively smooth model.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/adaboost_3_100.png){height=8cm}`\end{center}`{=latex}
::::
:::







## 2nd regularization strategy: random forests


`\begin{center}\vspace*{.0cm}`{=latex}

![](images/trees/bootstrap.pdf){ width=100% }

Parallel fits on **bootstrap** samples of the original dataset. \
The final model is the **average** of a **forest** of independent trees.

`\end{center}`{=latex}



## 2nd regularization strategy: random forests

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/forest_3_1.png){height=5cm}`\vspace*{.5cm}`{=latex}

**1 tree** of depth 3:\
a simple decision tree, \
computed on a bootstrap \
**subset** of the original sample.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/forest_3_1.png){height=8cm}`\end{center}`{=latex}
::::
:::


## 2nd regularization strategy: random forests

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/forest_3_2.png){height=5cm}`\vspace*{.5cm}`{=latex}

**2 trees** of depth 3:\
average of two decision trees, \
fitted on two independent\
bootstrap samples.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/forest_3_2.png){height=8cm}`\end{center}`{=latex}
::::
:::


## 2nd regularization strategy: random forests

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/forest_3_5.png){height=5cm}`\vspace*{.5cm}`{=latex}

**5 trees** of depth 3:\
average of five decision trees, \
fitted on five independent \
bootstrap samples. 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/forest_3_5.png){height=8cm}`\end{center}`{=latex}
::::
:::


## 2nd regularization strategy: random forests

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/forest_3_100.png){height=5cm}`\vspace*{.5cm}`{=latex}

**100 trees** of depth 3:\
a regularized decision rule. \
The model still follows the **axes** \
of the feature space, but is **smoother**.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/forest_3_100.png){height=8cm}`\end{center}`{=latex}
::::
:::





## Some features may require more work -- understand the context! `\cite{withings}`{=latex}

`\begin{center}\vspace*{.25cm}`{=latex}

![](images/features/bmi-chart.png){ height=70% }

The $~\textbf{Body Mass Index}~~=~~\text{weight}~~/~~\text{height}^\text{2}~$ \
is a good indicator for many health problems.

`\end{center}`{=latex}


## Some features may require more work -- understand the context!

`\begin{center}\vspace*{.25cm}`{=latex}

![](images/features/insee_richesse.png){ height=70% }

Applying thresholds on **postal codes** is mostly useless. \
Other statistics may be much more informative.

`\end{center}`{=latex}



## Some features may require more work -- understand the context!

`\begin{center}\vspace*{.0cm}`{=latex}

![](images/features/datetime.png){ height=70% }

Applying thresholds on **UNIX timestamps** is mostly useless. \
We must first apply periodic transforms to get hours-days-months.

`\end{center}`{=latex}



## Sometimes, the input features are just not good enough `\cite{heart_raw}`{=latex}

`\begin{center}\vspace*{.0cm}`{=latex}

![](images/voisinages/heart_naif.png){ height=70% }

Tree models cannot process **raw pixel values**. \
Standard radiomic features only take you so far.

`\end{center}`{=latex}


## Trees and forests -- conclusion

**Tree-based** models are:

- Highly `\BR{interpretable}`{=latex}.
- Well suited to **high-quality heterogeneous** features.
- **Easy** to use: XGBoost, LightGBM, scikit-learn...


`\begin{center}\vspace*{.0cm}`{=latex}

\
On the other hand, they produce **non-smooth** results \
and are biased along the **axes** of the feature space.\
\
\
This is a major limitation if you work with **homogeneous** features: \
the 3D xyz coordinates, pixel values, audio signals...
`\end{center}`{=latex}


# K-Nearest Neighbors

## The Euclidean metric is isotropic

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/knn_euclidean/axes.pdf){height=5cm}`\vspace*{.5cm}`{=latex}

**Thresholding** features promotes\
decisions along the **axes** \
of the feature space. 

`\end{center}`{=latex}
::::

:::: {.column width=50%}
`\begin{center}`{=latex}![](images/knn_euclidean/euclidean.pdf){height=5cm}`\vspace*{.5cm}`{=latex}

The squared **Euclidean** metric \
$\|(\text{x}_\text{1}, \dots, \text{x}_\text{D})\|^2 ~=~ \text{x}_\text{1}^\text{2} + \dots + \text{x}_\text{D}^\text{2}$ \
is invariant to **rotations**.

`\end{center}`{=latex}
::::

:::



## Average value among the K-Nearest Neighbors

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/knn_1.png){height=5cm}`\vspace*{.5cm}`{=latex}

With **K = 1** neighbor, we retrieve \
a simple nearest neighbor interpolation. \
This model is piecewise constant \
on the **Voronoi diagram** of x.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/knn_1.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Average value among the K-Nearest Neighbors

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/knn_2.png){height=5cm}`\vspace*{.5cm}`{=latex}

With **K = 2** neighbors, \
the cells of the diagram \
become smaller.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/knn_2.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Average value among the K-Nearest Neighbors

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/knn_3.png){height=5cm}`\vspace*{.5cm}`{=latex}

With **K = 3** neighbors, \
the cells of the diagram \
become smaller.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/knn_3.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Average value among the K-Nearest Neighbors

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/knn_4.png){height=5cm}`\vspace*{.5cm}`{=latex}

With **K = 4** neighbors, \
the cells of the diagram \
become smaller.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/knn_4.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Average value among the K-Nearest Neighbors

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/knn_5.png){height=5cm}`\vspace*{.5cm}`{=latex}

With **K = 5** neighbors, \
the model looks **smoother** and smoother \
but is still **piecewise constant**.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/knn_5.png){height=8cm}`\end{center}`{=latex}
::::
:::


## K-Nearest Neighbors: the main selling points

K-NN models are:

- `\BR{Interpretable}`{=latex}. \
  
- **Isotropic** -- which may or may *not* be a good thing! \

- **Easy** to deploy.

- **Fast**, parallel and GPU-friendly -- see our MVA Lecture 7 on algorithms.

- **Well-packaged** and scalable: FAISS, KeOps, (big-)ann-benchmarks.com...


## Major weakness: K-NNs require a good scaling of the input features

`\begin{center}`{=latex}
Unlike tree-based models, the Euclidean distance is sensitive \
to the **precise values** of the features x. 

Out-of-the-box, K-NNs are not even robust to the `\BR{choice of the units}`{=latex}\
for the columns of our dataset!

`\end{center}`{=latex}

We must **normalize** the input features using:

- A feature-wise rescaling using e.g. the **standard deviation**.

- A **multivariate** normalization using e.g. Principal Component Analysis. \
  The Euclidean distance with a normalized PCA is known as the **Mahalanobis** metric.

- Alternatively, a robust **equalization** of the feature histograms.

## Summary on trees and K-NNs

**Tree-based** and **K-NN** models are:

- `\BR{Interpretable}`{=latex} methods with **heterogeneous** / **homogeneous features**.

- Well-understood, well-packaged and easy to deploy.

- Excellent baselines for `\BR{interpolation}`{=latex}.

Unfortunately, both methods :

- Produce **non-smooth**, piecewise constant decision rules.

- Are `\BR{local}`{=latex} and do not estimate global trends. \
  They are not a natural fit for `\BR{extrapolation}`{=latex}, **forecasting**.


# Linear regression 


## A simple model: linear regression

`\begin{center}\vspace*{.5cm}`{=latex}

![](images/dataset/linear_regression.pdf){ height=65% }

We choose the weights **a**, **b**, $\dots$, **f** \
by minimizing a least squares error. \
 \

`\end{center}`{=latex}


```{=latex}
\end{frame}

\begin{frame}{A toy dataset, in dimension~1}
	\begin{figure}
		\centering
		\includegraphics<1>[width=.7\linewidth]{images/regression/dataset/data_1.png}%
		\includegraphics<2>[width=.7\linewidth]{images/regression/dataset/data_2.png}%
		\includegraphics<3>[width=.7\linewidth]{images/regression/dataset/data_3.png}%
		\includegraphics<4>[width=.7\linewidth]{images/regression/dataset/data_4.png}%
		\includegraphics<5>[width=.7\linewidth]{images/regression/dataset/data_5.png}%
		\includegraphics<6>[width=.7\linewidth]{images/regression/dataset/data_6.png}%
		\includegraphics<7>[width=.7\linewidth]{images/regression/dataset/data_7.png}%
		\includegraphics<8>[width=.7\linewidth]{images/regression/dataset/data_8.png}%
		\includegraphics<9-9>[width=.7\linewidth]{images/regression/dataset/data_9.png} %
	\end{figure}
\end{frame}

\begin{frame}{Iterative least squares fitting}
	\begin{figure}
		\centering
		\includegraphics<1>[width=.7\linewidth]{images/regression/poly_1/plot_0.png}%
		\includegraphics<2>[width=.7\linewidth]{images/regression/poly_1/plot_1.png}%
		\includegraphics<3>[width=.7\linewidth]{images/regression/poly_1/plot_2.png}%
		\includegraphics<4>[width=.7\linewidth]{images/regression/poly_1/plot_3.png}%
		\includegraphics<5>[width=.7\linewidth]{images/regression/poly_1/plot_5.png}%
		\includegraphics<6>[width=.7\linewidth]{images/regression/poly_1/plot_10.png}%
		\includegraphics<7>[width=.7\linewidth]{images/regression/poly_1/plot_20.png}%
		\includegraphics<8>[width=.7\linewidth]{images/regression/poly_1/plot_50.png}%
		\includegraphics<9>[width=.7\linewidth]{images/regression/poly_1/plot_100.png}%
	\end{figure}
\end{frame}

\begin{frame}{Linear regression}
```

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/linear.png){height=5cm}`\vspace*{.5cm}`{=latex}

Linear regression models \
a **monotonic trend**. \
It cannot handle complex relationships \
between the input x and the ouput y.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/linear.png){height=8cm}`\end{center}`{=latex}
::::
:::



## What should we do if the problem is complex?


`\begin{center}\vspace*{.5cm}`{=latex}

![](images/cakes/cakes.png){ width=95% }

 \
`\LARGE{ Take a break :-)}`{=latex}
`\end{center}`{=latex}


# Neural networks

## Maybe, we could introduce some intermediate variables in our model?


`\begin{center}\vspace*{.5cm}`{=latex}

![](images/regression/two_variables.png){ height=65% }

Domain experts may have suggested a **step-by-step** process \
to compute the quantity of interest -- say, the perimeter of an organ. \
 \

`\end{center}`{=latex}


```{=latex}
\end{frame}

\begin{frame}{Let's complexify our model with intermediate variables...}
\begin{figure}
\centering
\includegraphics<1>[width=\linewidth,height=.9\textheight]{images/regression/linear_2/plot_1.png}%
%\includegraphics<2>[width=\linewidth,height=.9\textheight]{images/regression/linear_2/plot_5.png}%
%\includegraphics<3>[width=\linewidth,height=.9\textheight]{images/regression/linear_2/plot_20.png}%
%\includegraphics<4>[width=\linewidth,height=.9\textheight]{images/regression/linear_2/plot_50.png}%
%\includegraphics<5>[width=\linewidth,height=.9\textheight]{images/regression/linear_2/plot_100.png}%
\includegraphics<2>[width=\linewidth,height=.9\textheight]{images/regression/linear_2/plot_169.png}%
\end{figure}
\end{frame}

\begin{frame}{Let's complexify our model with intermediate variables... and non-linearities!}
\begin{figure}
\centering
%\includegraphics<1>[width=\linewidth,height=.9\textheight]{images/regression/single_2/plot_1.png}%
%\includegraphics<2>[width=\linewidth,height=.9\textheight]{images/regression/single_2/plot_5.png}%
\includegraphics<1>[width=\linewidth,height=.9\textheight]{images/regression/single_2/plot_10.png}%
\includegraphics<2>[width=\linewidth,height=.9\textheight]{images/regression/single_2/plot_20.png}%
\includegraphics<3>[width=\linewidth,height=.9\textheight]{images/regression/single_2/plot_50.png}%
\includegraphics<4>[width=\linewidth,height=.9\textheight]{images/regression/single_2/plot_100.png}%
\includegraphics<5>[width=\linewidth,height=.9\textheight]{images/regression/single_2/plot_194.png}%
\end{figure}
\end{frame}

\begin{frame}{We can then increase the number of ``neurons''...}
\begin{figure}
\centering
\includegraphics<1>[width=\linewidth,height=.9\textheight]{images/regression/single_4/plot_1.png}%
\includegraphics<2>[width=\linewidth,height=.9\textheight]{images/regression/single_4/plot_5.png}%
\includegraphics<3>[width=\linewidth,height=.9\textheight]{images/regression/single_4/plot_10.png}%
\includegraphics<4>[width=\linewidth,height=.9\textheight]{images/regression/single_4/plot_20.png}%
\includegraphics<5>[width=\linewidth,height=.9\textheight]{images/regression/single_4/plot_50.png}%
\includegraphics<6>[width=\linewidth,height=.9\textheight]{images/regression/single_4/plot_100.png}%
\includegraphics<7>[width=\linewidth,height=.9\textheight]{images/regression/single_4/plot_173.png}%
\end{figure}
\end{frame}

\begin{frame}{We can then increase the number of ``neurons''... and layers!}
\begin{figure}
\centering
\includegraphics<1>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_0.png}%
%\includegraphics<2>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_2.png}%
%\includegraphics<3>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_3.png}%
%\includegraphics<4>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_50.png}%
%\includegraphics<5>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_100.png}%
\includegraphics<2>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_200.png}%
\includegraphics<3>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_500.png}%
\includegraphics<4>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_1000.png}%
\includegraphics<5>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_2000.png}%
\includegraphics<6>[width=\linewidth,height=.9\textheight]{images/regression/dual_100x100/plot_5000.png}%
\end{figure}
\end{frame}
```


## Fully connected neural networks, aka. multi-layer perceptrons

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/mlp_1.png){height=5cm}`\vspace*{.5cm}`{=latex}

MLP with **1 hidden neuron**. \
This is a piecewise linear model \
with at most 1 hinge. \
The optimizer doesn't use them all.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/mlp_1.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Fully connected neural networks, aka. multi-layer perceptrons

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/mlp_10.png){height=5cm}`\vspace*{.5cm}`{=latex}

MLP with **10 hidden neurons**. \
This is a piecewise linear model \
with at most 10 hinges. \
The optimizer doesn't use them all.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/mlp_10.png){height=8cm}`\end{center}`{=latex}
::::
:::

## Fully connected neural networks, aka. multi-layer perceptrons

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/mlp_20.png){height=5cm}`\vspace*{.5cm}`{=latex}

MLP with **20 hidden neurons**. \
This is a piecewise linear model \
with at most 20 hinges. \
The optimizer doesn't use them all.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/mlp_20.png){height=8cm}`\end{center}`{=latex}
::::
:::

## Fully connected neural networks, aka. multi-layer perceptrons

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/mlp_50.png){height=5cm}`\vspace*{.5cm}`{=latex}

MLP with **50 hidden neurons**. \
This is a piecewise linear model \
with at most 50 hinges. \
The optimizer doesn't use them all.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/mlp_50.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Fully connected neural networks, aka. multi-layer perceptrons

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/mlp_100.png){height=5cm}`\vspace*{.5cm}`{=latex}

MLP with **100 hidden neurons**. \
This is a piecewise linear model \
with at most 100 hinges. \
The optimizer doesn't use them all.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/mlp_100.png){height=8cm}`\end{center}`{=latex}
::::
:::





## Fully connected neural networks, aka. multi-layer perceptrons

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/mlp_100_100.png){height=5cm}`\vspace*{.5cm}`{=latex}

Deeper MLP with 2 hidden layers and \
**100 + 100 hidden neurons**, \
i.e. at most 100 x 100 hinges. \
The **non-convex, stochastic** optimization is \
**unreliable** and not reproducible.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/mlp_100_100.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Fully connected neural networks, aka. multi-layer perceptrons

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/mlp_100_100_100.png){height=5cm}`\vspace*{.5cm}`{=latex}

Deeper MLP with 3 hidden layers and \
**100 + 100 + 100 hidden neurons**, \
i.e. at most 100 x 100 x 100 hinges. \
The **non-convex, stochastic** optimization is \
**unreliable** and not reproducible.
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/mlp_100_100_100.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Fully connected neural networks, aka. multi-layer perceptrons

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/mlp_100_100_100_100.png){height=5cm}`\vspace*{.5cm}`{=latex}

Deeper MLP with 4 hidden layers and \
**100 + 100 + 100 + 100 hidden neurons**, \
i.e. at most 100 x 100 x 100 x 100 hinges. \
 \
Starting to look like a smooth origami ;-)
 

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/mlp_100_100_100_100.png){height=8cm}`\end{center}`{=latex}
::::
:::


## (Vanilla, fully connected) neural networks -- strengths and weaknesses

- `\BR{Modular}`{=latex} and easy to extend.
- Simplest way of implementing **high-dimensional piecewise linear** models.
- Extremely **well-supported** on CPU and GPU: PyTorch, TensorFlow...

`\begin{center}`{=latex}
Unfortunately, the optimization of the "neural" weights \
corresponds to a `\BR{non-convex}`{=latex} optimization problem. \
\
We must rely on **non-deterministic**, stochastic solvers. \
Performance and smoothness are **not** simply correlated to \
the number of neurons and layers.\
\
In most applications, this lack of reproducibility \
and interpretability is a **deal-breaker**.
`\end{center}`{=latex}

# Polynomial interpolation



## Polynomial interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_polynomial_d0.png){height=5cm}`\vspace*{.5cm}`{=latex}

Constant polynomials of **degree 0**: \
**D=1** -- 1 constant. \
**D=2** -- 1 constant.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_polynomial_d0.png){height=8cm}`\end{center}`{=latex}
::::
:::




## Polynomial interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_polynomial_d1.png){height=5cm}`\vspace*{.5cm}`{=latex}

Linear polynomials of **degree 1**: \
**D=1** -- 1, x. \
**D=2** -- 1, x, y.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_polynomial_d1.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Polynomial interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_polynomial_d2.png){height=5cm}`\vspace*{.5cm}`{=latex}

Quadratic polynomials of **degree 2**: \
**D=1** -- 1, x, $\text{x}^{\text{2}}$. \
**D=2** -- 1, x, y, $\text{x}^{\text{2}}$, xy, $\text{y}^{\text{2}}$.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_polynomial_d2.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Polynomial interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_polynomial_d3.png){height=5cm}`\vspace*{.5cm}`{=latex}

Cubic polynomials of **degree 3**: \
**D=1** -- 1, x, $\text{x}^{\text{2}}$, $\text{x}^{\text{3}}$. \
**D=2** -- 1, x, y, $\text{x}^{\text{2}}$, xy, $\text{y}^{\text{2}}$, $\text{x}^{\text{3}}$, $\text{x}^{\text{2}}$y, x$\text{y}^{\text{2}}$, $\text{y}^{\text{3}}$.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_polynomial_d3.png){height=8cm}`\end{center}`{=latex}
::::
:::




## Polynomial interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_polynomial_d4.png){height=5cm}`\vspace*{.5cm}`{=latex}

Quartic polynomials of **degree 4**: \
**D=1** -- 1, x, $\text{x}^{\text{2}}$, $\text{x}^{\text{3}}$, $\text{x}^{\text{4}}$. \
**D=2** -- 1, x, y, $\text{x}^{\text{2}}$, xy, $\text{y}^{\text{2}}$, $\text{x}^{\text{3}}$, $\text{x}^{\text{2}}$y, x$\text{y}^{\text{2}}$, $\text{y}^{\text{3}}$, \
$\text{x}^{\text{4}}$, $\text{x}^{\text{3}}$y, $\text{x}^{\text{2}}\text{y}^{\text{2}}$, x$\text{y}^{\text{3}}$, $\text{y}^{\text{4}}$. \
Starting to **overfit** in dimension D=2.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_polynomial_d4.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Polynomial interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_polynomial_d5.png){height=5cm}`\vspace*{.5cm}`{=latex}

Polynomials of **degree 5**: \
**D=1** -- 1, x, $\text{x}^{\text{2}}$, $\text{x}^{\text{3}}$, $\text{x}^{\text{4}}$, $\text{x}^{\text{5}}$. \
**D=2** -- 1, x, y, $\text{x}^{\text{2}}$, xy, $\text{y}^{\text{2}}$, $\text{x}^{\text{3}}$, $\text{x}^{\text{2}}$y, x$\text{y}^{\text{2}}$, $\text{y}^{\text{3}}$, \
$\text{x}^{\text{4}}$, $\text{x}^{\text{3}}$y, ..., $\text{y}^{\text{4}}$,
$\text{x}^{\text{5}}$, $\text{x}^{\text{4}}$y, ..., $\text{y}^{\text{5}}$. \
Full **overfit** in dimension D=2.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_polynomial_d5.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Polynomial interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_polynomial_d10.png){height=5cm}`\vspace*{.5cm}`{=latex}

Polynomials of **degree 10**: \
**D=1** -- 1, x, $\text{x}^{\text{2}}$, $\text{x}^{\text{3}}$, $\text{x}^{\text{4}}$, $\text{x}^{\text{5}}$, ..., $\text{x}^{\text{10}}$. \
**D=2** -- 1, x, y, $\text{x}^{\text{2}}$, xy, $\text{y}^{\text{2}}$, $\text{x}^{\text{3}}$, $\text{x}^{\text{2}}$y, x$\text{y}^{\text{2}}$, $\text{y}^{\text{3}}$, \
$\text{x}^{\text{4}}$, $\text{x}^{\text{3}}$y, ..., $\text{y}^{\text{4}}$,
$\text{x}^{\text{5}}$, $\text{x}^{\text{4}}$y, ..., $\text{y}^{\text{5}}$, ..., $\text{y}^{\text{10}}$. \
Full **overfit** in both examples.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_polynomial_d10.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Summary of the models that we have seen so far

"Non-parametric" methods:

- `\BR{Tree-based}`{=latex} models -- robust, but with a bias along the axes.
- `\BR{K-Nearest Neighbors}`{=latex} models -- isotropic, but requires a good scaling.


"Parametric" methods:

- `\BR{Linear}`{=latex} regression -- useful, but often too simplistic.
- `\BR{Neural networks}`{=latex} -- expressive, but unreliable.

`\BR{Polynomial}`{=latex} regression:

- Linear regression with polynomial features.
- Quadratic regression is fine -- but we badly **overfit** beyond degree 4-5.



# Kernel interpolation

## User-specified smoothness

Let's specify directly a linear parametric form for the model:

$$\text{F}(\text{a}_\text{1}, \dots, \text{a}_\text{J} ; \text{x}) ~=~ \text{a}_\text{1} \text{F}_\text{1}(\text{x}) ~+~ \cdots ~+~ \text{a}_\text{J} \text{F}_\text{J}(\text{x}).$$

In practice, we often use:

$$\text{F}(\text{a}_\text{j} ; \text{x}) ~=~ \sum_{\text{j}} \text{a}_\text{j}\, \text{k}(\text{x} - \text{x}_\text{j})$$

and say that $~~\text{k}(\text{x} - \text{y})~~$ is the `\BR{kernel}`{=latex} of our method.

## Some common kernels


`\begin{center}\vspace*{.0cm}`{=latex}

![](images/kernels.pdf){ height=70% }

\
Two main criteria: is the kernel **smooth** or **peaky**? \
Does the kernel have **compact support** or a **heavy tail**?

`\end{center}`{=latex}


## How do we choose the weights?

First method -- just use a fraction instead of a linear combination:

$$\text{F}(\text{x}) ~=~ \frac{\sum_{\text{j}} \text{k}(\text{x} - \text{x}_\text{j}) \, \text{y}_\text{j}}{\sum_{\text{j}} \text{k}(\text{x} - \text{x}_\text{j})}$$


`\begin{center}`{=latex}
The Nadaraya--Watson method assumes that $~~\text{k}(\text{x}-\text{y})~~$ takes positive values. \
\
It corresponds to a **barycentric interpolation** between the values $\text{y}_\text{j}$,\
with weights that are proportional to $~~\text{k}(\text{x} - \text{x}_\text{j})$.

`\end{center}`{=latex}
## Nadaraya--Watson interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/nw_gaussian_g02.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Smooth**, local Gaussian kernel with **σ = 0.2** \
$\text{k}(\text{x},\text{y}) = \text{exp} (- \|\text{x}-\text{y}\|^{\text{2}} / \text{2} \text{σ}^{\text{2}})$. \
\
**Smooth** local averaging \
on the unit interval and square.


`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/nw_gaussian_g02.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Nadaraya--Watson interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/nw_gaussian_g01.png){height=5cm}`\vspace*{.5cm}`{=latex}

Smooth, **local** Gaussian kernel with **σ = 0.1** \
$\text{k}(\text{x},\text{y}) = \text{exp} (- \|\text{x}-\text{y}\|^{\text{2}} / \text{2} \text{σ}^{\text{2}})$. \
\
**Sharper**, K-NN-like decision boundaries.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/nw_gaussian_g01.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Nadaraya--Watson interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/nw_cauchy_g01.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Heavy-tail** Cauchy kernel with **σ = 0.1** \
$\text{k}(\text{x},\text{y}) = \text{1}~/~(\text{1} + \|\text{x}-\text{y}\|^{\text{2}} / \text{σ}^{\text{2}})$. \
\
**Dampened** towards the global average.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/nw_cauchy_g01.png){height=8cm}`\end{center}`{=latex}
::::
:::




## Nadaraya--Watson interpolation

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/nw_laplace_g01.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Pointy** exponential kernel with **σ = 0.1** \
$\text{k}(\text{x},\text{y}) = \text{exp} (- \|\text{x}-\text{y}\| / \text{σ} )$. \
\
**Closer fit** to the training data.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/nw_laplace_g01.png){height=8cm}`\end{center}`{=latex}
::::
:::





## Nadaraya--Watson--Shepard interpolation -- Inverse Distance Weighting

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/nw_distance_g-1.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Singular** Shepard kernel \
$\text{k}(\text{x},\text{y}) = \text{1}~/~\|\text{x}-\text{y}\|$. \
\
**Perfect fit** to the training data.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/nw_distance_g-1.png){height=8cm}`\end{center}`{=latex}
::::
:::




## Nadaraya--Watson--Shepard interpolation -- Inverse Distance Weighting

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/nw_distance_g-05.png){height=5cm}`\vspace*{.5cm}`{=latex}

Singular and **heavy-tail** Shepard kernel \
$\text{k}(\text{x},\text{y}) = \text{1}~/~\sqrt{\|\text{x}-\text{y}\|}$. \
\
**Perfect fit** to the training data, \
dampening to the average value elsewhere.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/nw_distance_g-05.png){height=8cm}`\end{center}`{=latex}
::::
:::




## Nadaraya--Watson--Shepard interpolation -- Inverse Distance Weighting

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/nw_distance_g-2.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Highly singular** Shepard kernel \
$\text{k}(\text{x},\text{y}) = \text{1}~/~\|\text{x}-\text{y}\|^{\text{2}}$. \
\
**Perfect fit** to the training data, \
close to a **linear** interpolation.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/nw_distance_g-2.png){height=8cm}`\end{center}`{=latex}
::::
:::



## Nadaraya--Watson--Shepard interpolation -- Inverse Distance Weighting

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/nw_distance_g-4.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Very highly singular** Shepard kernel \
$\text{k}(\text{x},\text{y}) = \text{1}~/~\|\text{x}-\text{y}\|^{\text{4}}$. \
\
**Perfect fit** to the training data, \
close to a **nearest neighbor** interpolation.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/nw_distance_g-4.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Second method: solve a linear system


```{=latex}
\begin{alignat*}{12}
\text{F}(\text{x}_\text{1}) 
~&=~ \text{\textbf{a}}_\text{\textbf{1}} \, \varphi(\text{x}_\text{1} -\text{x}_\text{1})
~&+~& \cdots 
~&+~& \text{\textbf{a}}_\text{\textbf{N}}\, \varphi(\text{x}_\text{1} -\text{x}_\text{N})
~&\simeq~& \text{y}_\text{1} \\[.25cm]
\text{F}(\text{x}_\text{2}) 
~&=~ \text{\textbf{a}}_\text{\textbf{1}}\, \varphi(\text{x}_\text{2} -\text{x}_\text{1})
~&+~& \cdots 
~&+~& \text{\textbf{a}}_\text{\textbf{N}}\, \varphi(\text{x}_\text{2} -\text{x}_\text{N})
~&\simeq~& \text{y}_\text{2} \\[.25cm]
\vdots~~~
~&=~ ~~~~~~~~~\vdots
~&+~& \ddots 
~&+~& ~~~~~~~~~\vdots
~&\simeq~& \vdots \\[.25cm]
\text{F}(\text{x}_\text{N}) 
~&=~ \text{\textbf{a}}_\text{\textbf{1}}\, \varphi(\text{x}_\text{N} -\text{x}_\text{1})
~&+~& \cdots 
~&+~& \text{\textbf{a}}_\text{\textbf{N}}\, \varphi(\text{x}_\text{N} -\text{x}_\text{N})
~&\simeq~& \text{y}_\text{N}\\
\end{alignat*}
```


`\begin{center}`{=latex}
Linear system $\Phi \, \textbf{a} = \text{y}$
`\end{center}`{=latex}


## Approximate kernel regression

Enforcing a **perfect fit** to the data may not be reasonable.

Instead, we target a trade-off between accuracy and smoothness:
$$\min_\text{a} ~~\|\Phi \text{a} - \text{y}\|^\text{2} + \text{Reg}(\text{a}).$$

Popular regularization terms are **convex**:

- **Ridge**: $α \|\text{a}\|^\text{2} = α ( \text{a}_\text{1}^\text{2} + \cdots + \text{a}_\text{N}^\text{2} ).$

- **Lasso**: $\lambda \|\text{a}\|_\text{1} = \lambda ( |\text{a}_\text{1}| + \cdots + |\text{a}_\text{N}|).$

- **Elastic Net**: $\lambda \|\text{a}\|_\text{1} + α \|\text{a}\|^\text{2}.$


## Kernel ridge regression

```{=latex}
\begin{align*}
\min_\text{a}~ \|\Phi\,\text{a} - \text{y}\|^2 ~+~ α\,\|\text{a}\|^2 
~~&=~~ (\text{a}^\intercal \Phi^\intercal - \text{y}^\intercal)(\Phi \text{a} - \text{y})
+ α\, \text{a}^\intercal \text{a} \\
~~&=~~ \text{a}^\intercal ( \Phi^\intercal \Phi + α\,\text{Id}_\text{N}  )\, \text{a}
- 2\,\text{y}^\intercal \Phi \text{a}
+ \text{y}^\intercal \text{y} \\[.5cm]
\Longrightarrow ~~ \text{a} ~~&=~~
( \Phi^\intercal \Phi + α\,\text{Id}_\text{N}  )^{-\text{1}}\, \Phi^\intercal \text{y} \\
&=~~ \Phi^\intercal
( \Phi \Phi^\intercal + α\,\text{Id}_\text{N}  )^{-\text{1}}  \text{y} \\[.5cm]
\Longrightarrow ~~
\text{F}(\text{x}) ~~&=~~
\Phi\,\text{a} ~=~ \Phi \Phi^\intercal
( \Phi \Phi^\intercal + α\,\text{Id}_\text{N}  )^{-\text{1}} \, \text{y}~.
\end{align*}

\begin{center}
A fundamental object appears: \\
the symmetric, positive, semidefinite \BR{kernel matrix} $~~\text{K} =  \Phi \Phi^\intercal$.
\end{center}
```

## The kernel trick

$$\text{K} =  \Phi \Phi^\intercal ~~~~\text{i.e.}~~~~
\text{K}(\text{x}_\text{i},\text{x}_\text{j})
~=~ 
\langle ~\Phi(\text{x}_\text{i}),  \Phi(\text{x}_\text{j})~\rangle 
~=~
\sum_{\text{s}=\text{1}}^\text{N} \varphi(\text{x}_\text{i} - \text{x}_\text{s})\,\varphi(\text{x}_\text{j} - \text{x}_\text{s}) $$

This may be **expensive**: N terms for every coefficient of K.

Fortunately, we may use the continuous limit instead:

$$\text{k}(\text{x}_\text{i},\text{x}_\text{j}) ~=~ \int_{\text{x}} \varphi(\text{x}_\text{i} - \text{x})\,\varphi(\text{x}_\text{j} - \text{x})\,\text{dx}$$

This dot product between **two translated copies** of $\varphi$ is often known in `\BR{closed form}`{=latex}.


## Computational perspective

`\begin{center}`{=latex}

We consider functions $~~\text{k}(\text{x}_\text{i} - \text{x}_\text{j})~~$ that can be written \
as the previous integral for a suitable function $\varphi$. \
\
Criterion: if the **Fourier** transform $~~\widehat{\text{k}}(\omega)~~$ is real-valued and positive, \
then $~~\widehat{\varphi}(\omega) = \sqrt{\widehat{\text{k}}(\omega)}~~$ works.

Then, **kernel ridge regression** simply relies on the model: \
$\text{F}(\text{x})~=~ \text{K}\, (\text{K} + α\,\text{Id}_\text{N})^{-\text{1}} \,\text{y}$.

`\end{center}`{=latex}

On GPUs, we may solve this linear system efficiently using:

- **KeOps** -- bruteforce methods scale to N $=$ 1,000,000 in seconds.
- **FalkonML** -- approximate methods scale to N $=$ 1,000,000,000 in hours.


## Kriging, Gaussian process regression `\cite{leclercq2018bayesian}`{=latex}

`\begin{center}`{=latex}

![](images/applications/GP_regression.png){height=45%}

Kernel ridge regression has a **rich history** in applied mathematics. \
It is especially popular in geostatistics to estimate smooth terrain models:\
the approximation parameter α controls the **nugget** effect.\
\
This theory is also behind **Sobolev** norms and **Gaussian** processes...\
More about this in the MVA Lecture 6 on probability distributions!
`\end{center}`{=latex}


## Kernel ridge regression -- with α = 0.1

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_gaussian_a01_s1.png){height=5cm}`\vspace*{.5cm}`{=latex}

Smooth, **global** Gaussian kernel with **σ = 1.0** \
$\text{k}(\text{x},\text{y}) = \text{exp} (- \|\text{x}-\text{y}\|^{\text{2}} / \text{2} \text{σ}^{\text{2}})$. \
\
Only models a **global** linear trend.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_gaussian_a01_s1.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Kernel ridge regression -- with α = 0.1

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_gaussian_a01_s05.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Smooth** Gaussian kernel with **σ = 0.5** \
$\text{k}(\text{x},\text{y}) = \text{exp} (- \|\text{x}-\text{y}\|^{\text{2}} / \text{2} \text{σ}^{\text{2}})$. \
\
Starts to discern **different regions**.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_gaussian_a01_s05.png){height=8cm}`\end{center}`{=latex}
::::
:::

## Kernel ridge regression -- with α = 0.1

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_gaussian_a01_s02.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Smooth** Gaussian kernel with **σ = 0.2** \
$\text{k}(\text{x},\text{y}) = \text{exp} (- \|\text{x}-\text{y}\|^{\text{2}} / \text{2} \text{σ}^{\text{2}})$. \
\
Well-suited to the **sampling density**.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_gaussian_a01_s02.png){height=8cm}`\end{center}`{=latex}
::::
:::

## Kernel ridge regression -- with α = 0.1

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_gaussian_a01_s01.png){height=5cm}`\vspace*{.5cm}`{=latex}

Smooth, **local** Gaussian kernel with **σ = 0.1** \
$\text{k}(\text{x},\text{y}) = \text{exp} (- \|\text{x}-\text{y}\|^{\text{2}} / \text{2} \text{σ}^{\text{2}})$. \
\
**Overfits** on individual sample values.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_gaussian_a01_s01.png){height=8cm}`\end{center}`{=latex}
::::
:::




## Kernel ridge regression -- with α = 0.1

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_cauchy_a01_s02.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Heavy-tail** Cauchy kernel with **σ = 0.2** \
$\text{k}(\text{x},\text{y}) = \text{1}~/~(\text{1} + \|\text{x}-\text{y}\|^{\text{2}} / \text{σ}^{\text{2}})$. \
\
**Extrapolates** with more confidence.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_cauchy_a01_s02.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Kernel ridge regression -- with α = 0.1

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_laplace_a01_s02.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Pointy** exponential kernel with **σ = 0.2** \
$\text{k}(\text{x},\text{y}) = \text{exp} (- \|\text{x}-\text{y}\| / \text{σ} )$. \
\
**Closer fit** to the training data.

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_laplace_a01_s02.png){height=8cm}`\end{center}`{=latex}
::::
:::


## Kernel ridge regression -- with α = 0.1

::: columns
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/1D_bowl/kernel_distance_a01_g1.png){height=5cm}`\vspace*{.5cm}`{=latex}

**Pointy**, **global** distance kernel \
$\text{k}(\text{x},\text{y}) = - \|\text{x}-\text{y}\|$. \
\
Models both **local** and **global** trends. \
**Excellent parameter-free baseline.**

`\end{center}`{=latex}
::::
:::: {.column width=50%}
`\begin{center}`{=latex}![](images/output_regression/2D_n100/kernel_distance_a01_g1.png){height=8cm}`\end{center}`{=latex}
::::
:::




# Conclusion

## Numerous regression models... But what about the curse of dimensionality?

`\begin{center}\vspace*{.25cm}`{=latex}

![](images/conclusion.pdf){ width=90% }

`\end{center}`{=latex}

# References

